@@ -1,13 +1,28 @@
-import numpy as np
-from typing import List, Optional, Union, Dict, Any
 from .compat import pd_DataFrame, pd_Series, pd_read_parquet
+import numpy as np
 import blosc
 import dill
 
-MeanMatchType = Union[int, float, Dict[str, float], Dict[str, int]]
-VarSchemType = Optional[Union[List[str], Dict[str, List[str]]]]
-VarParamType = Dict[Union[str, int], Dict[str, Any]]
-CatFeatType = Union[str, List[str], List[int], Dict[int, dict]]
+
+_REGRESSIVE_OBJECTIVES = [
+    "regression",
+    "regression_l1",
+    "poisson",
+    "huber",
+    "fair",
+    "mape",
+    "cross_entropy",
+    "cross_entropy_lambda" "quantile",
+    "tweedie",
+    "gamma",
+]
+
+
+_CATEGORICAL_OBJECTIVES = [
+    "binary",
+    "multiclass",
+    "multiclassova",
+]
 
 
 def ampute_data(
@@ -73,6 +88,74 @@ def ampute_data(
     return amputed_data
 
 
+def stratified_subset(y, size, groups, cat, seed):
+    """
+    Subsample y using stratification. y is divided into quantiles,
+    and then elements are randomly chosen from each quantile to
+    come up with the subsample.
+
+    Parameters
+    ----------
+    y: np.ndarray
+        The variable to use for stratification
+    size: int
+        How large the subset should be
+    groups: int
+        How many groups to break y into. The more groups, the more
+        balanced (but less random) y will be
+    cat: bool
+        Is y already categorical? If so, we can skip the group creation
+    seed: int
+        The random seed to use.
+
+    Returns
+    -------
+    The indices of y that have been chosen.
+
+    """
+    rs = np.random.RandomState(seed)
+
+    if isinstance(y, pd_Series):
+        if y.dtype.name == "category":
+            y = y.cat.codes
+        y = y.values
+
+    if cat:
+        digits = y
+    else:
+        q = [x / groups for x in range(1, groups)]
+        bins = np.quantile(y, q)
+        digits = np.digitize(y, bins, right=True)
+
+    digits_v, digits_c = np.unique(digits, return_counts=True)
+    digits_i = np.arange(digits_v.shape[0])
+    digits_p = digits_c / digits_c.sum()
+    digits_s = (digits_p * size).round(0).astype("int32")
+    diff = size - digits_s.sum()
+    if diff != 0:
+        digits_fix = rs.choice(digits_i, size=abs(diff), p=digits_p, replace=False)
+        if diff < 0:
+            for d in digits_fix:
+                digits_s[d] -= 1
+        else:
+            for d in digits_fix:
+                digits_s[d] += 1
+
+    sub = np.zeros(shape=size).astype("int32")
+    added = 0
+    for d_i in digits_i:
+        d_v = digits_v[d_i]
+        n = digits_s[d_i]
+        ind = np.where(digits == d_v)[0]
+        choice = rs.choice(ind, size=n, replace=False)
+        sub[added : (added + n)] = choice
+        added += n
+
+    sub.sort()
+
+    return sub
+
+
 def stratified_continuous_folds(y, nfold):
     """
     Create primitive stratified folds for continuous data.
@@ -127,8 +210,15 @@ def hash_int32(x):
     return x
 
 
+def _draw_random_int32(random_state, size):
+    nums = random_state.randint(
+        low=0, high=np.iinfo("int32").max, size=size, dtype="int32"
+    )
+    return nums
+
+
 def ensure_rng(
-    random_state: Optional[Union[int, np.random.RandomState]] = None
+    random_state
 ) -> np.random.RandomState:
     """
     Creates a random number generator based on an optional seed.  This can be
@@ -145,6 +235,21 @@ def ensure_rng(
 
 
 def load_kernel(filepath, n_threads=None):
+    """
+    Loads a kernel that was saved using save_kernel().
+
+    Parameters
+    ----------
+    filepath: str
+        The filepath of the saved kernel
+
+    n_threads: int
+        The threads to use for decompression. By default, all threads are used.
+
+    Returns
+    -------
+
+    """
     n_threads = blosc.detect_number_of_cores() if n_threads is None else n_threads
     blosc.set_nthreads(n_threads)
     with open(filepath, "rb") as f:
@@ -266,166 +371,11 @@ def _subset_data(dat, row_ind=None, col_ind=None, return_1d=False):
         raise ValueError("Unknown data class passed.")
 
 
-# Not all aliases are stored somewhere retrievable in lightgbm..
-# So they are manually stored here.
-# List can be found at:
-# https://github.com/microsoft/LightGBM/blob/master/src/io/config_auto.cpp
-param_mapping = {
-    "config_file": "config",
-    "task_type": "task",
-    "objective_type": "objective",
-    "app": "objective",
-    "application": "objective",
-    "boosting_type": "boosting",
-    "boost": "boosting",
-    "train": "data",
-    "train_data": "data",
-    "train_data_file": "data",
-    "data_filename": "data",
-    "test": "valid",
-    "valid_data": "valid",
-    "valid_data_file": "valid",
-    "test_data": "valid",
-    "test_data_file": "valid",
-    "valid_filenames": "valid",
-    "num_iteration": "num_iterations",
-    "n_iter": "num_iterations",
-    "num_tree": "num_iterations",
-    "num_trees": "num_iterations",
-    "num_round": "num_iterations",
-    "num_rounds": "num_iterations",
-    "num_boost_round": "num_iterations",
-    "n_estimators": "num_iterations",
-    "shrinkage_rate": "learning_rate",
-    "eta": "learning_rate",
-    "num_leaf": "num_leaves",
-    "max_leaves": "num_leaves",
-    "max_leaf": "num_leaves",
-    "tree": "tree_learner",
-    "tree_type": "tree_learner",
-    "tree_learner_type": "tree_learner",
-    "num_thread": "num_threads",
-    "nthread": "num_threads",
-    "nthreads": "num_threads",
-    "n_jobs": "num_threads",
-    "device": "device_type",
-    "random_seed": "seed",
-    "random_state": "seed",
-    "hist_pool_size": "histogram_pool_size",
-    "min_data_per_leaf": "min_data_in_leaf",
-    "min_data": "min_data_in_leaf",
-    "min_child_samples": "min_data_in_leaf",
-    "min_sum_hessian_per_leaf": "min_sum_hessian_in_leaf",
-    "min_sum_hessian": "min_sum_hessian_in_leaf",
-    "min_hessian": "min_sum_hessian_in_leaf",
-    "min_child_weight": "min_sum_hessian_in_leaf",
-    "sub_row": "bagging_fraction",
-    "subsample": "bagging_fraction",
-    "bagging": "bagging_fraction",
-    "pos_sub_row": "pos_bagging_fraction",
-    "pos_subsample": "pos_bagging_fraction",
-    "pos_bagging": "pos_bagging_fraction",
-    "neg_sub_row": "neg_bagging_fraction",
-    "neg_subsample": "neg_bagging_fraction",
-    "neg_bagging": "neg_bagging_fraction",
-    "subsample_freq": "bagging_freq",
-    "bagging_fraction_seed": "bagging_seed",
-    "sub_feature": "feature_fraction",
-    "colsample_bytree": "feature_fraction",
-    "sub_feature_bynode": "feature_fraction_bynode",
-    "colsample_bynode": "feature_fraction_bynode",
-    "extra_tree": "extra_trees",
-    "early_stopping_rounds": "early_stopping_round",
-    "early_stopping": "early_stopping_round",
-    "n_iter_no_change": "early_stopping_round",
-    "max_tree_output": "max_delta_step",
-    "max_leaf_output": "max_delta_step",
-    "reg_alpha": "lambda_l1",
-    "reg_lambda": "lambda_l2",
-    "lambda": "lambda_l2",
-    "min_split_gain": "min_gain_to_split",
-    "rate_drop": "drop_rate",
-    "topk": "top_k",
-    "mc": "monotone_constraints",
-    "monotone_constraint": "monotone_constraints",
-    "monotone_constraining_method": "monotone_constraints_method",
-    "mc_method": "monotone_constraints_method",
-    "monotone_splits_penalty": "monotone_penalty",
-    "ms_penalty": "monotone_penalty",
-    "mc_penalty": "monotone_penalty",
-    "feature_contrib": "feature_contri",
-    "fc": "feature_contri",
-    "fp": "feature_contri",
-    "feature_penalty": "feature_contri",
-    "fs": "forcedsplits_filename",
-    "forced_splits_filename": "forcedsplits_filename",
-    "forced_splits_file": "forcedsplits_filename",
-    "forced_splits": "forcedsplits_filename",
-    "verbose": "verbosity",
-    "model_input": "input_model",
-    "model_in": "input_model",
-    "model_output": "output_model",
-    "model_out": "output_model",
-    "save_period": "snapshot_freq",
-    "linear_trees": "linear_tree",
-    "subsample_for_bin": "bin_construct_sample_cnt",
-    "data_seed": "data_random_seed",
-    "is_sparse": "is_enable_sparse",
-    "enable_sparse": "is_enable_sparse",
-    "sparse": "is_enable_sparse",
-    "is_enable_bundle": "enable_bundle",
-    "bundle": "enable_bundle",
-    "is_pre_partition": "pre_partition",
-    "two_round_loading": "two_round",
-    "use_two_round_loading": "two_round",
-    "has_header": "header",
-    "label": "label_column",
-    "weight": "weight_column",
-    "group": "group_column",
-    "group_id": "group_column",
-    "query_column": "group_column",
-    "query": "group_column",
-    "query_id": "group_column",
-    "ignore_feature": "ignore_column",
-    "blacklist": "ignore_column",
-    "cat_feature": "categorical_feature",
-    "categorical_column": "categorical_feature",
-    "cat_column": "categorical_feature",
-    "is_save_binary": "save_binary",
-    "is_save_binary_file": "save_binary",
-    "is_predict_raw_score": "predict_raw_score",
-    "predict_rawscore": "predict_raw_score",
-    "raw_score": "predict_raw_score",
-    "is_predict_leaf_index": "predict_leaf_index",
-    "leaf_index": "predict_leaf_index",
-    "is_predict_contrib": "predict_contrib",
-    "contrib": "predict_contrib",
-    "predict_result": "output_result",
-    "prediction_result": "output_result",
-    "predict_name": "output_result",
-    "prediction_name": "output_result",
-    "pred_name": "output_result",
-    "name_pred": "output_result",
-    "convert_model_file": "convert_model",
-    "num_classes": "num_class",
-    "unbalance": "is_unbalance",
-    "unbalanced_sets": "is_unbalance",
-    "metrics": "metric",
-    "metric_types": "metric",
-    "output_freq": "metric_freq",
-    "training_metric": "is_provide_training_metric",
-    "is_training_metric": "is_provide_training_metric",
-    "train_metric": "is_provide_training_metric",
-    "ndcg_eval_at": "eval_at",
-    "ndcg_at": "eval_at",
-    "map_eval_at": "eval_at",
-    "map_at": "eval_at",
-    "num_machine": "num_machines",
-    "local_port": "local_listen_port",
-    "port": "local_listen_port",
-    "machine_list_file": "machine_list_filename",
-    "machine_list": "machine_list_filename",
-    "mlist": "machine_list_filename",
-    "workers": "machines",
-    "nodes": "machines",
-}
+def logodds(probability):
+    odds_ratio = probability / (1 - probability)
+    log_odds = np.log(odds_ratio)
+    return log_odds
+
+
+def logistic_function(log_odds):
+    return 1 / (1 + np.exp(-log_odds))
